@@ -30,7 +30,7 @@
 #include <libtracker-common/tracker-file-utils.h>
 #include <libtracker-common/tracker-ontology.h>
 
-#include <tracker-fts/tracker-fts.h>
+#include <libtracker-fts/tracker-fts.h>
 
 #include <libtracker-db/tracker-db-manager.h>
 #include <libtracker-db/tracker-db-dbus.h>
@@ -297,11 +297,6 @@ ensure_resource_id (const gchar *uri)
 		tracker_db_statement_execute (stmt, NULL);
 		g_object_unref (stmt);
 
-		stmt = tracker_db_interface_create_statement (iface, "INSERT INTO \"fts\" (rowid) VALUES (?)");
-		tracker_db_statement_bind_int (stmt, 0, id);
-		tracker_db_statement_execute (stmt, NULL);
-		g_object_unref (stmt);
-
 		g_hash_table_insert (update_buffer.resource_cache, g_strdup (uri), GUINT_TO_POINTER (id));
 	}
 
@@ -426,21 +421,20 @@ tracker_data_update_buffer_flush (void)
 		TrackerProperty *prop;
 		GValueArray *values;
 
-		fts = g_string_new ("");
+		tracker_fts_update_init (update_buffer.id);
 
 		g_hash_table_iter_init (&iter, update_buffer.predicates);
 		while (g_hash_table_iter_next (&iter, (gpointer*) &prop, (gpointer*) &values)) {
 			if (tracker_property_get_fulltext_indexed (prop)) {
+				fts = g_string_new ("");
 				for (i = 0; i < values->n_values; i++) {
 					g_string_append (fts, g_value_get_string (g_value_array_get_nth (values, i)));
 					g_string_append_c (fts, ' ');
 				}
+				tracker_fts_update_text (update_buffer.id, tracker_data_query_resource_id (tracker_property_get_uri (prop)), fts->str);
+				g_string_free (fts, TRUE);
 			}
 		}
-
-		tracker_fts_update_init (update_buffer.id);
-		tracker_fts_update_text (update_buffer.id, 0, fts->str);
-		g_string_free (fts, TRUE);
 	}
 
 	g_hash_table_remove_all (update_buffer.predicates);
@@ -513,14 +507,11 @@ tracker_data_blank_buffer_flush (void)
 static void
 cache_create_service_decomposed (TrackerClass           *cl)
 {
-	TrackerDBInterface *iface;
 	TrackerClass       **super_classes;
 	GValue              gvalue = { 0 };
 	gint                i;
 	const gchar        *class_uri;
 
-	iface = tracker_db_manager_get_db_interface ();
-
 	/* also create instance of all super classes */
 	super_classes = tracker_class_get_super_classes (cl);
 	while (*super_classes) {
@@ -806,9 +797,6 @@ cache_set_metadata_decomposed (TrackerProperty	*property,
 		g_value_init (&gvalue, G_TYPE_INT);
 		g_value_set_int (&gvalue, object_id);
 		break;
-	case TRACKER_PROPERTY_TYPE_BLOB:
-	case TRACKER_PROPERTY_TYPE_STRUCT:
-	case TRACKER_PROPERTY_TYPE_FULLTEXT:
 	default:
 		return;
 	}
@@ -907,9 +895,6 @@ delete_metadata_decomposed (gint resource_id,
 		object_id = ensure_resource_id (value);
 		tracker_db_statement_bind_int (stmt, 1, object_id);
 		break;
-	case TRACKER_PROPERTY_TYPE_BLOB:
-	case TRACKER_PROPERTY_TYPE_STRUCT:
-	case TRACKER_PROPERTY_TYPE_FULLTEXT:
 	default:
 		g_assert_not_reached ();
 	}
@@ -1112,13 +1097,6 @@ tracker_data_delete_statement (const gchar            *subject,
 			tracker_db_statement_execute (stmt, NULL);
 			g_object_unref (stmt);
 
-			if (strcmp (tracker_class_get_name (class), "rdfs:Resource") == 0) {
-				stmt = tracker_db_interface_create_statement (iface, "DELETE FROM \"fts\" WHERE rowid = ?");
-				tracker_db_statement_bind_int (stmt, 0, subject_id);
-				tracker_db_statement_execute (stmt, NULL);
-				g_object_unref (stmt);
-			}
-
 			/* delete rows from class tables */
 			delete_resource_type (subject_id, class);
 
@@ -1239,6 +1217,8 @@ tracker_data_insert_statement (const gchar            *subject,
 		} else {
 			tracker_data_insert_statement_with_string (subject, predicate, object, error);
 		}
+	} else if (strcmp (predicate, TRACKER_PREFIX "uri") == 0) {
+		tracker_data_insert_statement_with_uri (subject, predicate, object, error);
 	} else {
 		g_set_error (error, TRACKER_DATA_ERROR, TRACKER_DATA_ERROR_UNKNOWN_PROPERTY,
 		             "Property '%s' not found in the ontology", predicate);
@@ -1261,9 +1241,13 @@ tracker_data_insert_statement_with_uri (const gchar            *subject,
 
 	property = tracker_ontology_get_property_by_uri (predicate);
 	if (property == NULL) {
-		g_set_error (error, TRACKER_DATA_ERROR, TRACKER_DATA_ERROR_UNKNOWN_PROPERTY,
-		             "Property '%s' not found in the ontology", predicate);
-		return;
+		if (strcmp (predicate, TRACKER_PREFIX "uri") == 0) {
+			/* virtual tracker:uri property */
+		} else {
+			g_set_error (error, TRACKER_DATA_ERROR, TRACKER_DATA_ERROR_UNKNOWN_PROPERTY,
+				     "Property '%s' not found in the ontology", predicate);
+			return;
+		}
 	} else if (tracker_property_get_data_type (property) != TRACKER_PROPERTY_TYPE_RESOURCE) {
 		g_set_error (error, TRACKER_DATA_ERROR, TRACKER_DATA_ERROR_INVALID_TYPE,
 		             "Property '%s' does not accept URIs", predicate);
@@ -1736,6 +1720,7 @@ tracker_data_delete_resource_description (const gchar *uri)
 				}
 			}
 
+			single_cursor = NULL;
 			if (!first) {
 				g_string_append_printf (sql, " FROM \"%s\" WHERE ID = ?", tracker_class_get_name (class));
 				stmt = tracker_db_interface_create_statement (iface, "%s", sql->str);
