@@ -78,6 +78,7 @@ static GObjectClass *parent_class = NULL;
 static void  tracker_metadata_tile_class_init	    (TrackerMetadataTileClass *class);
 static void  tracker_metadata_tile_init		    (TrackerMetadataTile      *tile);
 static gboolean tracker_metadata_tile_expose_event(GtkWidget *widget, GdkEventExpose *event);
+#if 0
 static void tracker_metadata_tile_show (TrackerMetadataTile *tile);
 static void _property_to_label (GtkWidget *label, const char *prop, const char *string);
 static void _date_to_label (GtkWidget *label, const char *prop, const char *string);
@@ -87,11 +88,14 @@ static void _dimensions_to_label (GtkWidget *label, const char *width, const cha
 static void _seconds_to_label (GtkWidget *label, const char *prop, const char *string);
 static void _bitrate_to_label (GtkWidget *label, const char *prop, const char *string);
 static void _int_to_label (GtkWidget *label, const char *prop, const char *string);
+#endif
 
 static inline gboolean is_empty_string (const char *s);
 
 /* structs & enums */
 
+#if 0
+
 static const char *default_keys[] =
 {
 	"File:Name",
@@ -103,6 +107,8 @@ static const char *default_keys[] =
 	NULL
 };
 
+#endif
+
 enum {
 	DEFAULT_NAME,
 	DEFAULT_PATH,
@@ -113,6 +119,8 @@ enum {
 	DEFAULT_N_KEYS
 };
 
+#if 0
+
 static const char *doc_keys[] =
 {
 	"File:Name",
@@ -125,6 +133,8 @@ static const char *doc_keys[] =
 	NULL
 };
 
+#endif
+
 enum {
 	DOC_NAME,
 	DOC_SUBJECT,
@@ -136,7 +146,7 @@ enum {
 	DOC_N_KEYS
 };
 
-
+#if 0
 
 static const char *email_keys[] =
 {
@@ -149,6 +159,8 @@ static const char *email_keys[] =
 	NULL
 };
 
+#endif
+
 enum {
 	EMAIL_SENDER,
 	EMAIL_SUBJECT,
@@ -159,6 +171,8 @@ enum {
 	EMAIL_N_KEYS
 };
 
+#if 0
+
 static const char *webhistory_keys[] =
 {
 	"Doc:URL",
@@ -169,6 +183,8 @@ static const char *webhistory_keys[] =
 	NULL
 };
 
+#endif
+
 enum {
 	WEBHISTORY_URL,
 	WEBHISTORY_TITLE,
@@ -178,6 +194,7 @@ enum {
 	WEBHISTORY_N_KEYS
 };
 
+#if 0
 
 static const char *app_keys[] =
 {
@@ -188,6 +205,8 @@ static const char *app_keys[] =
 	NULL
 };
 
+#endif
+
 enum {
 	APP_DISPLAYNAME,
 	APP_GENERIC_NAME,
@@ -196,6 +215,7 @@ enum {
 	APP_N_KEYS
 };
 
+#if 0
 
 static const char *audio_keys[] =
 {
@@ -211,6 +231,8 @@ static const char *audio_keys[] =
 	NULL
 };
 
+#endif
+
 enum {
 	AUDIO_TITLE,
 	AUDIO_ARTIST,
@@ -224,6 +246,8 @@ enum {
 	AUDIO_N_KEYS
 };
 
+#if 0
+
 static const char *image_keys[] =
 {
 	"File:Name",
@@ -239,6 +263,8 @@ static const char *image_keys[] =
 	NULL
 };
 
+#endif
+
 enum {
 	IMAGE_TITLE,
 	IMAGE_HEIGHT,
@@ -253,6 +279,8 @@ enum {
 	IMAGE_N_KEYS
 };
 
+#if 0
+
 static const char *video_keys[] =
 {
 	"File:Name",
@@ -266,6 +294,8 @@ static const char *video_keys[] =
 	NULL
 };
 
+#endif
+
 enum {
 	VIDEO_TITLE,
 	VIDEO_HEIGHT,
@@ -284,6 +314,7 @@ is_empty_string (const char *s)
 	return s == NULL || s[0] == '\0';
 }
 
+#if 0
 
 static void
 _show_labels (TrackerMetadataTile *tile, gboolean label_visible)
@@ -403,7 +434,6 @@ _tile_tracker_populate_email (char **array, GError *error, TrackerMetadataTile *
 	gtk_widget_show (priv->info2);
 }
 
-
 static void
 _tile_tracker_populate_applications (char **array, GError *error, TrackerMetadataTile *tile)
 {
@@ -440,11 +470,8 @@ _tile_tracker_populate_applications (char **array, GError *error, TrackerMetadat
 	_show_labels (tile, FALSE);
 	gtk_widget_show (priv->info1);
 	gtk_widget_show (priv->info2);
-
 }
 
-
-
 static void
 _tile_tracker_populate_audio (char **array, GError *error, TrackerMetadataTile *tile)
 {
@@ -520,9 +547,6 @@ _tile_tracker_populate_audio (char **array, GError *error, TrackerMetadataTile *
 	_size_to_label ( priv->info5, array[AUDIO_SIZE] , _("Size : <b>%s</b>"));
 	_property_to_label ( priv->info6, array[AUDIO_CODEC] , _("Codec : <b>%s</b>"));
 
-
-
-
 	/* free properties */
 	g_strfreev (array);
 
@@ -591,9 +615,6 @@ _tile_tracker_populate_image (char **array, GError *error, TrackerMetadataTile *
 	_property_to_label ( priv->info5, array[IMAGE_FOCAL] , _("Focal Length : <b>%s</b>"));
 	_property_to_label ( priv->info6, array[IMAGE_EXPO] , _("Exposure Time : <b>%s</b>"));
 
-
-
-
 	tracker_metadata_tile_show (tile);
 	g_strfreev (array);
 }
@@ -623,11 +644,8 @@ _tile_tracker_populate_video (char **array, GError *error, TrackerMetadataTile *
 	_property_to_label ( priv->info5, array[VIDEO_CODEC] , _("Encoded In : <b>%s</b>"));
 	_property_to_label ( priv->info6, array[VIDEO_FRAMERATE] , _("Framerate : <b>%s</b>"));
 
-
-
 	tracker_metadata_tile_show (tile);
 	g_strfreev (array);
-
 }
 
 /* populates the metadata tile for a document */
@@ -656,12 +674,10 @@ _tile_tracker_populate_documents (char **array, GError *error, TrackerMetadataTi
 	_date_to_label ( priv->info5, array[DOC_CREATED] , _("Created : <b>%s</b>"));
 	_property_to_label ( priv->info6, array[DOC_COMMENTS] , _("Comments : <b>%s</b>"));
 
-
 	tracker_metadata_tile_show (tile);
 	g_strfreev (array);
 }
 
-
 /*populates the metadata tile for a web history url */
 static void
 _tile_tracker_populate_webhistory(char **array, GError *error, TrackerMetadataTile *tile )
@@ -690,11 +706,8 @@ _tile_tracker_populate_webhistory(char **array, GError *error, TrackerMetadataTi
 	_show_labels (tile, FALSE);
 	gtk_widget_show (priv->info1);
 	gtk_widget_show (priv->info2);
-
 }
 
-
-
 /* UTILILTY FUNCTIONS FOR CONVERSIONS */
 
 /* Converts bitrate to kbs */
@@ -913,7 +926,7 @@ _property_to_label (GtkWidget *label, const char *prop, const char *string)
 	}
 }
 
-
+#endif
 
 /**
  * tracker_metadata_tile_set_uri:
@@ -928,6 +941,8 @@ _property_to_label (GtkWidget *label, const char *prop, const char *string)
  *
  **/
 
+/* TODO: Port to SPARQL */
+#if 0
 void
 tracker_metadata_tile_set_uri (TrackerMetadataTile *tile, const gchar *uri,
 							  ServiceType service_type,
@@ -1029,7 +1044,6 @@ tracker_metadata_tile_set_uri (TrackerMetadataTile *tile, const gchar *uri,
 	}
 
 
-
 	if (uri) {
 		gtk_widget_show (priv->tag_bar);
 		tracker_tag_bar_set_uri (TRACKER_TAG_BAR (priv->tag_bar), service_type, uri);
@@ -1058,6 +1072,8 @@ tracker_metadata_tile_show (TrackerMetadataTile *tile)
 	}
 }
 
+#endif
+
 static gboolean
 tracker_metadata_tile_toggle_view (GtkWidget *button, TrackerMetadataTile *tile)
 {
@@ -1338,7 +1354,7 @@ tracker_metadata_tile_new (void)
 	tile = g_object_new (TRACKER_TYPE_METADATA_TILE, NULL);
 	priv = TRACKER_METADATA_TILE_GET_PRIVATE (tile);
 
-	client = tracker_connect (TRUE);
+	client = tracker_connect (TRUE, -1);
 	priv->client = client;
 	return tile;
 }
