@@ -28,6 +28,17 @@
 #include "tracker-monitor.h"
 #include "tracker-utils.h"
 
+/**
+ * SECTION:tracker-miner-fs
+ * @short_description: Abstract base class for filesystem miners
+ * @include: libtracker-miner/tracker-miner-fs.h
+ *
+ * #TrackerMinerFS is an abstract base class for miners that collect data
+ * from the filesystem, all the filesystem crawling and monitoring is
+ * abstracted away, leaving to implementations the decisions of what
+ * directories/files should it process, and the actual data extraction.
+ **/
+
 #define TRACKER_MINER_FS_GET_PRIVATE(o) (G_TYPE_INSTANCE_GET_PRIVATE ((o), TRACKER_TYPE_MINER_FS, TrackerMinerFSPrivate))
 
 typedef struct {
@@ -40,6 +51,12 @@ typedef struct {
 	gboolean  recurse;
 } DirectoryData;
 
+typedef struct {
+	GFile *file;
+	GCancellable *cancellable;
+	TrackerSparqlBuilder *builder;
+} ProcessData;
+
 struct TrackerMinerFSPrivate {
 	TrackerMonitor *monitor;
 	TrackerCrawler *crawler;
@@ -60,16 +77,17 @@ struct TrackerMinerFSPrivate {
 	guint           crawl_directories_id;
 	guint		item_queues_handler_id;
 
-	GFile          *current_file;
-	GCancellable   *cancellable;
-
 	gdouble         throttle;
 
+	GList          *processing_pool;
+	guint           pool_limit;
+
 	/* Status */
 	guint           been_started : 1;
 	guint           been_crawled : 1;
 	guint           shown_totals : 1;
 	guint           is_paused : 1;
+	guint           is_crawling : 1;
 
 	/* Statistics */
 	guint		total_directories_found;
@@ -96,13 +114,15 @@ enum {
 	CHECK_DIRECTORY,
 	CHECK_DIRECTORY_CONTENTS,
 	MONITOR_DIRECTORY,
+	PROCESS_FILE,
 	FINISHED,
 	LAST_SIGNAL
 };
 
 enum {
 	PROP_0,
-	PROP_THROTTLE
+	PROP_THROTTLE,
+	PROP_POOL_LIMIT
 };
 
 static void           fs_finalize                  (GObject        *object);
@@ -205,7 +225,24 @@ tracker_miner_fs_class_init (TrackerMinerFSClass *klass)
 							      "Modifier for the indexing speed, 0 is max speed",
 							      0, 1, 0,
 							      G_PARAM_READWRITE));
-
+	g_object_class_install_property (object_class,
+					 PROP_POOL_LIMIT,
+					 g_param_spec_uint ("process-pool-limit",
+							    "Processing pool limit",
+							    "Number of files that can be concurrently processed",
+							    1, G_MAXUINT, 1,
+							    G_PARAM_READWRITE | G_PARAM_CONSTRUCT));
+	/**
+	 * TrackerMinerFS::check-file:
+	 * @miner_fs: the #TrackerMinerFS
+	 * @file: a #GFile
+	 *
+	 * The ::check-file signal is emitted either on the filesystem crawling
+	 * phase or whenever a new file appears in a monitored directory
+	 * in order to check whether @file must be inspected my @miner_fs.
+	 *
+	 * Returns: %TRUE if @file must be inspected.
+	 **/
 	signals[CHECK_FILE] =
 		g_signal_new ("check-file",
 			      G_OBJECT_CLASS_TYPE (object_class),
@@ -215,6 +252,17 @@ tracker_miner_fs_class_init (TrackerMinerFSClass *klass)
 			      NULL,
 			      tracker_marshal_BOOLEAN__OBJECT,
 			      G_TYPE_BOOLEAN, 1, G_TYPE_FILE);
+	/**
+	 * TrackerMinerFS::check-directory:
+	 * @miner_fs: the #TrackerMinerFS
+	 * @directory: a #GFile
+	 *
+	 * The ::check-directory signal is emitted either on the filesystem crawling
+	 * phase or whenever a new directory appears in a monitored directory
+	 * in order to check whether @directory must be inspected my @miner_fs.
+	 *
+	 * Returns: %TRUE if @directory must be inspected.
+	 **/
 	signals[CHECK_DIRECTORY] =
 		g_signal_new ("check-directory",
 			      G_OBJECT_CLASS_TYPE (object_class),
@@ -224,6 +272,20 @@ tracker_miner_fs_class_init (TrackerMinerFSClass *klass)
 			      NULL,
 			      tracker_marshal_BOOLEAN__OBJECT,
 			      G_TYPE_BOOLEAN, 1, G_TYPE_FILE);
+	/**
+	 * TrackerMinerFS::check-directory-contents:
+	 * @miner_fs: the #TrackerMinerFS
+	 * @directory: a #GFile
+	 * @children: #GList of #GFile<!-- -->s
+	 *
+	 * The ::check-directory-contents signal is emitted either on the filesystem
+	 * crawling phase or whenever a new directory appears in a monitored directory
+	 * in order to check whether @directory must be inspected my @miner_fs based on
+	 * the directory contents, for some implementations this signal may be useful
+	 * to discard backup directories for example.
+	 *
+	 * Returns: %TRUE if @directory must be inspected.
+	 **/
 	signals[CHECK_DIRECTORY_CONTENTS] =
 		g_signal_new ("check-directory-contents",
 			      G_OBJECT_CLASS_TYPE (object_class),
@@ -233,6 +295,17 @@ tracker_miner_fs_class_init (TrackerMinerFSClass *klass)
 			      NULL,
 			      tracker_marshal_BOOLEAN__OBJECT_POINTER,
 			      G_TYPE_BOOLEAN, 2, G_TYPE_FILE, G_TYPE_POINTER);
+	/**
+	 * TrackerMinerFS::monitor-directory:
+	 * @miner_fs: the #TrackerMinerFS
+	 * @directory: a #GFile
+	 *
+	 * The ::monitor-directory is emitted either on the filesystem crawling phase
+	 * or whenever a new directory appears in a monitored directory in order to
+	 * check whether @directory must be monitored for filesystem changes or not.
+	 *
+	 * Returns: %TRUE if the directory must be monitored for changes.
+	 **/
 	signals[MONITOR_DIRECTORY] =
 		g_signal_new ("monitor-directory",
 			      G_OBJECT_CLASS_TYPE (object_class),
@@ -242,6 +315,50 @@ tracker_miner_fs_class_init (TrackerMinerFSClass *klass)
 			      NULL,
 			      tracker_marshal_BOOLEAN__OBJECT,
 			      G_TYPE_BOOLEAN, 1, G_TYPE_FILE);
+	/**
+	 * TrackerMinerFS::process-file:
+	 * @miner_fs: the #TrackerMinerFS
+	 * @file: a #GFile
+	 * @builder: a #TrackerSparqlBuilder
+	 * @cancellable: a #GCancellable
+	 *
+	 * The ::process-file signal is emitted whenever a file should
+	 * be processed, and it's metadata extracted.
+	 *
+	 * @builder is the #TrackerSparqlBuilder where all sparql updates
+	 * to be performed for @file will be appended.
+	 *
+	 * This signal allows both synchronous and asynchronous extraction,
+	 * in the synchronous case @cancellable can be safely ignored. In
+	 * either case, on successful metadata extraction, implementations
+	 * must call tracker_miner_fs_notify_file() to indicate that
+	 * processing has finished on @file, so the miner can execute
+	 * the SPARQL updates and continue processing other files.
+	 *
+	 * Returns: %TRUE if the file is accepted for processing,
+	 *          %FALSE if the file should be ignored.
+	 **/
+	signals[PROCESS_FILE] =
+		g_signal_new ("process-file",
+			      G_OBJECT_CLASS_TYPE (object_class),
+			      G_SIGNAL_RUN_LAST,
+			      G_STRUCT_OFFSET (TrackerMinerFSClass, process_file),
+			      NULL, NULL,
+			      tracker_marshal_BOOLEAN__OBJECT_OBJECT_OBJECT,
+			      G_TYPE_BOOLEAN,
+			      3, G_TYPE_FILE, TRACKER_TYPE_SPARQL_BUILDER, G_TYPE_CANCELLABLE),
+	/**
+	 * TrackerMinerFS::finished:
+	 * @miner_fs: the #TrackerMinerFS
+	 * @elapsed: elapsed time since mining was started
+	 * @directories_found: number of directories found
+	 * @directories_ignored: number of ignored directories
+	 * @files_found: number of files found
+	 * @files_ignored: number of ignored files
+	 *
+	 * The ::finished signal is emitted when @miner_fs has finished
+	 * all pending processing.
+	 **/
 	signals[FINISHED] =
 		g_signal_new ("finished",
 			      G_TYPE_FROM_CLASS (object_class),
@@ -312,6 +429,47 @@ tracker_miner_fs_init (TrackerMinerFS *object)
 	priv->quark_ignore_file = g_quark_from_static_string ("tracker-ignore-file");
 }
 
+static ProcessData *
+process_data_new (GFile                *file,
+		  GCancellable         *cancellable,
+		  TrackerSparqlBuilder *builder)
+{
+	ProcessData *data;
+
+	data = g_slice_new (ProcessData);
+	data->file = g_object_ref (file);
+	data->cancellable = g_object_ref (cancellable);
+	data->builder = g_object_ref (builder);
+
+	return data;
+}
+
+static void
+process_data_free (ProcessData *data)
+{
+	g_object_unref (data->file);
+	g_object_unref (data->cancellable);
+	g_object_unref (data->builder);
+	g_slice_free (ProcessData, data);
+}
+
+static ProcessData *
+process_data_find (TrackerMinerFS *fs,
+		   GFile          *file)
+{
+	GList *l;
+
+	for (l = fs->private->processing_pool; l; l = l->next) {
+		ProcessData *data = l->data;
+
+		if (g_file_equal (data->file, file)) {
+			return data;
+		}
+	}
+
+	return NULL;
+}
+
 static void
 fs_finalize (GObject *object)
 {
@@ -333,15 +491,14 @@ fs_finalize (GObject *object)
 	g_object_unref (priv->crawler);
 	g_object_unref (priv->monitor);
 
-	if (priv->cancellable) {
-		g_object_unref (priv->cancellable);
-	}
-
 	if (priv->directories) {
 		g_list_foreach (priv->directories, (GFunc) directory_data_free, NULL);
 		g_list_free (priv->directories);
 	}
 
+	g_list_foreach (priv->processing_pool, (GFunc) process_data_free, NULL);
+	g_list_free (priv->processing_pool);
+
 	g_queue_foreach (priv->items_moved, (GFunc) item_moved_data_free, NULL);
 	g_queue_free (priv->items_moved);
 
@@ -363,11 +520,17 @@ fs_set_property (GObject      *object,
 		 const GValue *value,
 		 GParamSpec   *pspec)
 {
+	TrackerMinerFS *fs = TRACKER_MINER_FS (object);
+
 	switch (prop_id) {
 	case PROP_THROTTLE:
 		tracker_miner_fs_set_throttle (TRACKER_MINER_FS (object),
 					       g_value_get_double (value));
 		break;
+	case PROP_POOL_LIMIT:
+		fs->private->pool_limit = g_value_get_uint (value);
+		g_message ("Miner process pool limit is set to %d", fs->private->pool_limit);
+		break;
 	default:
 		G_OBJECT_WARN_INVALID_PROPERTY_ID (object, prop_id, pspec);
 		break;
@@ -388,6 +551,9 @@ fs_get_property (GObject    *object,
 	case PROP_THROTTLE:
 		g_value_set_double (value, fs->private->throttle);
 		break;
+	case PROP_POOL_LIMIT:
+		g_value_set_uint (value, fs->private->pool_limit);
+		break;
 	default:
 		G_OBJECT_WARN_INVALID_PROPERTY_ID (object, prop_id, pspec);
 		break;
@@ -579,15 +745,13 @@ item_moved_data_free (ItemMovedData *data)
 }
 
 static void
-item_add_or_update_cb (TrackerMinerFS       *fs,
-		       GFile                *file,
-		       TrackerSparqlBuilder *sparql,
-		       const GError         *error,
-		       gpointer              user_data)
+item_add_or_update_cb (TrackerMinerFS *fs,
+		       ProcessData    *data,
+		       const GError   *error)
 {
 	gchar *uri;
 
-	uri = g_file_get_uri (file);
+	uri = g_file_get_uri (data->file);
 
 	if (error) {
 		g_message ("Could not process '%s': %s", uri, error->message);
@@ -597,7 +761,7 @@ item_add_or_update_cb (TrackerMinerFS       *fs,
 		g_debug ("Adding item '%s'", uri);
 
 		full_sparql = g_strdup_printf ("DROP GRAPH <%s> %s",
-					       uri, tracker_sparql_builder_get_result (sparql));
+					       uri, tracker_sparql_builder_get_result (data->builder));
 
 		tracker_miner_execute_batch_update (TRACKER_MINER (fs), full_sparql, NULL);
 		g_free (full_sparql);
@@ -610,24 +774,12 @@ item_add_or_update_cb (TrackerMinerFS       *fs,
 		}
 	}
 
-	if (fs->private->cancellable) {
-		g_object_unref (fs->private->cancellable);
-		fs->private->cancellable = NULL;
-	}
-
-	if (fs->private->current_file) {
-		g_object_unref (fs->private->current_file);
-		fs->private->current_file = NULL;
-	}
-
-	/* Can be NULL on error */
-	if (sparql) {
-		g_object_unref (sparql);
-	}
+	fs->private->processing_pool =
+		g_list_remove (fs->private->processing_pool, data);
+	process_data_free (data);
 
 	g_free (uri);
 
-	/* Processing is now done, continue with other files */
 	item_queue_handlers_set_up (fs);
 }
 
@@ -635,33 +787,60 @@ static gboolean
 item_add_or_update (TrackerMinerFS *fs,
 		    GFile          *file)
 {
+	TrackerMinerFSPrivate *priv;
 	TrackerSparqlBuilder *sparql;
-	gboolean processing;
+	GCancellable *cancellable;
+	gboolean processing, retval;
+	ProcessData *data;
 
-	if (fs->private->cancellable) {
-		g_debug ("Cancellable for older operation still around, destroying");
-		g_object_unref (fs->private->cancellable);
-	}
+	priv = fs->private;
+	retval = TRUE;
 
-	fs->private->cancellable = g_cancellable_new ();
+	cancellable = g_cancellable_new ();
 	sparql = tracker_sparql_builder_new_update ();
+	g_object_ref (file);
 
-	processing = TRACKER_MINER_FS_GET_CLASS (fs)->process_file (fs, file, sparql,
-								    fs->private->cancellable,
-								    item_add_or_update_cb,
-								    NULL);
+	data = process_data_new (file, cancellable, sparql);
+	priv->processing_pool = g_list_prepend (priv->processing_pool, data);
+
+	g_signal_emit (fs, signals[PROCESS_FILE], 0,
+		       file, sparql, cancellable,
+		       &processing);
 
 	if (!processing) {
-		g_object_unref (sparql);
-		g_object_unref (fs->private->cancellable);
-		fs->private->cancellable = NULL;
+		/* Re-fetch data, since it might have been
+		 * removed in broken implementations
+		 */
+		data = process_data_find (fs, file);
 
-		return TRUE;
+		if (!data) {
+			gchar *uri;
+
+			uri = g_file_get_uri (file);
+			g_critical ("%s has returned FALSE in ::process-file for '%s', "
+				    "but it seems that this file has been processed through "
+				    "tracker_miner_fs_notify_file(), this is an "
+				    "implementation error", G_OBJECT_TYPE_NAME (fs), uri);
+			g_free (uri);
+		} else {
+			priv->processing_pool = g_list_remove (priv->processing_pool, data);
+			process_data_free (data);
+		}
 	} else {
-		fs->private->current_file = g_object_ref (file);
+		guint length;
+
+		length = g_list_length (priv->processing_pool);
+
+		if (length >= priv->pool_limit) {
+			retval = FALSE;
+		}
 	}
 
-	return FALSE;
+	g_object_unref (file);
+	g_object_unref (cancellable);
+	g_object_unref (sparql);
+
+	return retval;
 }
 
 static gboolean
@@ -691,7 +870,8 @@ static void
 item_remove (TrackerMinerFS *fs,
 	     GFile          *file)
 {
-	gchar *sparql, *uri;
+	GString *sparql;
+	gchar *uri, *slash_uri;
 
 	uri = g_file_get_uri (file);
 
@@ -703,16 +883,34 @@ item_remove (TrackerMinerFS *fs,
 		return;
 	}
 
-	/* Delete resource */
-	sparql = g_strdup_printf ("DELETE { <%s> a rdfs:Resource }", uri);
-	tracker_miner_execute_batch_update (TRACKER_MINER (fs), sparql, NULL);
-	g_free (sparql);
+	if (!g_str_has_suffix (uri, "/")) {
+		slash_uri = g_strconcat (uri, "/", NULL);
+	} else {
+		slash_uri = g_strdup (uri);
+	}
 
-	/* FIXME: Should delete recursively? */
+	sparql = g_string_new ("");
+
+	/* Delete all children */
+	g_string_append_printf (sparql,
+				"DELETE { ?u a rdfs:Resource } "
+				"WHERE { ?u nfo:belongsToContainer ?p . FILTER (fn:starts-with (?p, \"%s\")) } ",
+				slash_uri);
+
+	/* Delete resource itself */
+	g_string_append_printf (sparql,
+				"DELETE { <%s> a rdfs:Resource }",
+				uri);
+
+	tracker_miner_execute_batch_update (TRACKER_MINER (fs), sparql->str, NULL);
+
+	g_string_free (sparql, TRUE);
+	g_free (slash_uri);
+	g_free (uri);
 }
 
 static void
-update_file_uri_recursively (TrackerMinerFS *fs,
+item_update_uri_recursively (TrackerMinerFS *fs,
 			     GString        *sparql_update,
 			     const gchar    *source_uri,
 			     const gchar    *uri)
@@ -724,8 +922,7 @@ update_file_uri_recursively (TrackerMinerFS *fs,
 		 source_uri,
 		 uri);
 
-	/* FIXME: tracker:uri doesn't seem to exist */
-	/* g_string_append_printf (sparql_update, " <%s> tracker:uri <%s> .", source_uri, uri); */
+	g_string_append_printf (sparql_update, " <%s> tracker:uri <%s> .", source_uri, uri);
 
 	sparql = g_strdup_printf ("SELECT ?child WHERE { ?child nfo:belongsToContainer <%s> }", source_uri);
 	result_set = tracker_miner_execute_sparql (TRACKER_MINER (fs), sparql, NULL);
@@ -748,7 +945,7 @@ update_file_uri_recursively (TrackerMinerFS *fs,
 
 			child_uri = g_strdup_printf ("%s%s", uri, *child_source_uri + strlen (source_uri));
 
-			update_file_uri_recursively (fs, sparql_update, *child_source_uri, child_uri);
+			item_update_uri_recursively (fs, sparql_update, *child_source_uri, child_uri);
 
 			g_free (child_source_uri);
 			g_free (child_uri);
@@ -810,10 +1007,8 @@ item_move (TrackerMinerFS *fs,
 	escaped_filename = g_strescape (g_file_info_get_display_name (file_info), NULL);
 
 	g_string_append_printf (sparql, " <%s> nfo:fileName \"%s\" .", source_uri, escaped_filename);
-	g_string_append_printf (sparql, " <%s> nie:isStoredAs <%s> .", source_uri, uri);
 
-	/* FIXME: This function just seemed to update the thumbnail */
-	/* update_file_uri_recursively (fs, sparql, source_uri, uri); */
+	item_update_uri_recursively (fs, sparql, source_uri, uri);
 
 	g_string_append (sparql, " }");
 
@@ -913,6 +1108,10 @@ item_queue_handlers_cb (gpointer user_data)
 	fs = user_data;
 	queue = item_queue_get_next_file (fs, &file, &source_file);
 
+	if (!fs->private->timer) {
+		fs->private->timer = g_timer_new ();
+	}
+
 	/* Update progress, but don't spam it. */
 	g_get_current_time (&time_now);
 
@@ -925,7 +1124,10 @@ item_queue_handlers_cb (gpointer user_data)
 	switch (queue) {
 	case QUEUE_NONE:
 		/* Print stats and signal finished */
-		process_stop (fs);
+		if (!fs->private->is_crawling &&
+		    !fs->private->processing_pool) {
+			process_stop (fs);
+		}
 
 		/* No more files left to process */
 		keep_processing = FALSE;
@@ -996,8 +1198,9 @@ item_queue_handlers_set_up (TrackerMinerFS *fs)
 		return;
 	}
 
-	if (!fs->private->timer) {
-		fs->private->timer = g_timer_new ();
+	if (g_list_length (fs->private->processing_pool) >= fs->private->pool_limit) {
+		/* There is no room in the pool for more files */
+		return;
 	}
 
 	g_object_get (fs, "status", &status, NULL);
@@ -1127,22 +1330,13 @@ monitor_item_created_cb (TrackerMonitor *monitor,
 
 	if (should_process) {
 		if (is_directory) {
-			gboolean add_monitor = TRUE;
-
-			g_signal_emit (fs, signals[MONITOR_DIRECTORY], 0, file, &add_monitor);
-
-			if (add_monitor) {
-				tracker_monitor_add (fs->private->monitor, file);
-			}
-
-			/* Add to the list */
 			tracker_miner_fs_add_directory (fs, file, TRUE);
+		} else {
+			g_queue_push_tail (fs->private->items_created,
+					   g_object_ref (file));
+			
+			item_queue_handlers_set_up (fs);
 		}
-
-		g_queue_push_tail (fs->private->items_created,
-				   g_object_ref (file));
-
-		item_queue_handlers_set_up (fs);
 	}
 
 	g_free (path);
@@ -1241,15 +1435,19 @@ monitor_item_moved_cb (TrackerMonitor *monitor,
 	fs = user_data;
 
 	if (!is_source_monitored) {
-		gchar *path;
+		if (is_directory) {
+			gchar *path;
 
-		path = g_file_get_path (other_file);
+			path = g_file_get_path (other_file);
 
-#ifdef FIX
-		/* If the source is not monitored, we need to crawl it. */
-		tracker_crawler_add_unexpected_path (fs->private->crawler, path);
-#endif
-		g_free (path);
+			g_debug ("Not in store:'?'->'%s' (DIR) (move monitor event, source unknown)", 
+				 path);
+
+			/* If the source is not monitored, we need to crawl it. */
+			tracker_miner_fs_add_directory (fs, other_file, TRUE);
+
+			g_free (path);
+		}
 	} else {
 		gchar *path;
 		gchar *other_path;
@@ -1284,18 +1482,10 @@ monitor_item_moved_cb (TrackerMonitor *monitor,
 				
 				item_queue_handlers_set_up (fs);
 			} else {
-				gboolean add_monitor = TRUE;
-				
-				g_signal_emit (fs, signals[MONITOR_DIRECTORY], 0, file, &add_monitor);
-				
-				if (add_monitor) {
-					tracker_monitor_add (fs->private->monitor, file);	     
-				}
+				g_debug ("Not in store:'?'->'%s' (DIR) (move monitor event, source monitored)", 
+					 path);
 
-#ifdef FIX
-				/* If this is a directory we need to crawl it */
-				tracker_crawler_add_unexpected_path (fs->private->crawler, other_path);
-#endif
+				tracker_miner_fs_add_directory (fs, other_file, TRUE);
 			}
 		} else if (!should_process_other) {
 			/* Delete old file */
@@ -1403,6 +1593,8 @@ crawler_finished_cb (TrackerCrawler *crawler,
 		}
 	}
 
+	fs->private->is_crawling = FALSE;
+
 	/* Update stats */
 	fs->private->directories_found += directories_found;
 	fs->private->directories_ignored += directories_ignored;
@@ -1415,7 +1607,7 @@ crawler_finished_cb (TrackerCrawler *crawler,
 	fs->private->total_files_ignored += files_ignored;
 
 	g_message ("%s crawling files after %2.2f seconds",
-		   was_interrupted ? "Stoped" : "Finished",
+		   was_interrupted ? "Stopped" : "Finished",
 		   g_timer_elapsed (fs->private->timer, NULL));
 	g_message ("  Found %d directories, ignored %d directories",
 		   directories_found,
@@ -1453,7 +1645,6 @@ crawl_directories_cb (gpointer user_data)
 		return FALSE;
 	}
 
-
 	fs->private->current_directory = fs->private->directories->data;
 	fs->private->directories = g_list_remove (fs->private->directories,
 						  fs->private->current_directory);
@@ -1476,6 +1667,7 @@ crawl_directories_cb (gpointer user_data)
 				   fs->private->current_directory->file,
 				   fs->private->current_directory->recurse)) {
 		/* Crawler when restart the idle function when done */
+		fs->private->is_crawling = TRUE;
 		fs->private->crawl_directories_id = 0;
 		return FALSE;
 	}
@@ -1534,6 +1726,14 @@ crawl_directories_stop (TrackerMinerFS *fs)
 	}
 }
 
+/**
+ * tracker_miner_fs_add_directory:
+ * @fs: a #TrackerMinerFS
+ * @file: #GFile for the directory to inspect
+ * @recurse: whether the directory should be inspected recursively
+ *
+ * Tells the filesystem miner to inspect a directory.
+ **/
 void
 tracker_miner_fs_add_directory (TrackerMinerFS *fs,
 				GFile          *file,
@@ -1571,13 +1771,22 @@ check_files_removal (GQueue *queue,
 	}
 }
 
+/**
+ * tracker_miner_fs_remove_directory:
+ * @fs: a #TrackerMinerFS
+ * @file: #GFile for the directory to be removed
+ *
+ * Removes a directory from being inspected by @fs.
+ *
+ * Returns: %TRUE if the directory was successfully removed.
+ **/
 gboolean
 tracker_miner_fs_remove_directory (TrackerMinerFS *fs,
 				   GFile          *file)
 {
 	TrackerMinerFSPrivate *priv;
 	gboolean return_val = FALSE;
-	GList *dirs;
+	GList *dirs, *pool;
 
 	g_return_val_if_fail (TRACKER_IS_MINER_FS (fs), FALSE);
 	g_return_val_if_fail (G_IS_FILE (file), FALSE);
@@ -1619,19 +1828,30 @@ tracker_miner_fs_remove_directory (TrackerMinerFS *fs,
 	check_files_removal (priv->items_updated, file);
 	check_files_removal (priv->items_created, file);
 
-	if (priv->current_file &&
-	    priv->cancellable &&
-	    (g_file_equal (priv->current_file, file) ||
-	     g_file_has_prefix (priv->current_file, file))) {
-		/* Cancel processing if currently processed file is
-		 * inside the removed directory.
-		 */
-		g_cancellable_cancel (priv->cancellable);
+	pool = fs->private->processing_pool;
+
+	while (pool) {
+		ProcessData *data = pool->data;
+
+		if (g_file_equal (data->file, file) ||
+		    g_file_has_prefix (data->file, file)) {
+			g_cancellable_cancel (data->cancellable);
+		}
 	}
 
 	return return_val;
 }
 
+/**
+ * tracker_miner_fs_set_throttle:
+ * @fs: a #TrackerMinerFS
+ * @throttle: throttle value, between 0 and 1
+ *
+ * Tells the filesystem miner to throttle its operations.
+ * a value of 0 means no throttling at all, so the miner
+ * will perform operations at full speed, 1 is the slowest
+ * value.
+ **/
 void
 tracker_miner_fs_set_throttle (TrackerMinerFS *fs,
 			       gdouble         throttle)
@@ -1664,6 +1884,14 @@ tracker_miner_fs_set_throttle (TrackerMinerFS *fs,
 	}
 }
 
+/**
+ * tracker_miner_fs_get_throttle:
+ * @fs: a #TrackerMinerFS
+ *
+ * Gets the current throttle value. see tracker_miner_fs_set_throttle().
+ *
+ * Returns: current throttle value.
+ **/
 gdouble
 tracker_miner_fs_get_throttle (TrackerMinerFS *fs)
 {
@@ -1671,3 +1899,43 @@ tracker_miner_fs_get_throttle (TrackerMinerFS *fs)
 
 	return fs->private->throttle;
 }
+
+/**
+ * tracker_miner_fs_notify_file:
+ * @fs: a #TrackerMinerFS
+ * @file: a #GFile
+ * @error: a #GError with the error that happened during processing, or %NULL.
+ *
+ * Notifies @fs that all processing on @file has been finished, if any error
+ * happened during file data processing, it should be passed in @error, else
+ * that parameter will contain %NULL to reflect success.
+ **/
+void
+tracker_miner_fs_notify_file (TrackerMinerFS *fs,
+			      GFile          *file,
+			      const GError   *error)
+{
+	ProcessData *data;
+
+	g_return_if_fail (TRACKER_IS_MINER_FS (fs));
+	g_return_if_fail (G_IS_FILE (file));
+
+	data = process_data_find (fs, file);
+
+	if (!data) {
+		gchar *uri;
+
+		uri = g_file_get_uri (file);
+		g_critical ("%s has notified that file '%s' has been processed, "
+			    "but that file was not in the processing queue. "
+			    "This is an implementation error, please ensure that "
+			    "tracker_miner_fs_notify_file() is called on the right "
+			    "file and that the ::process-file signal didn't return "
+			    "FALSE for it", G_OBJECT_TYPE_NAME (fs), uri);
+		g_free (uri);
+
+		return;
+	}
+
+	item_add_or_update_cb (fs, data, error);
+}
