@@ -776,22 +776,28 @@ albumart_copy_to_local (TrackerStorage *hal,
 			const gchar    *filename, 
 			const gchar    *local_uri)
 {
-	GList *removable_roots, *l;
+	GSList *removable_roots, *l;
 	gboolean on_removable_device = FALSE;
 	guint flen;
 
-	flen = strlen (filename);
-
 	/* Determining if we are on a removable device */
 #ifdef HAVE_HAL
-	g_return_if_fail (hal != NULL);
+	if (!hal) {
+		/* This is usually because we are running on the
+		 * command line, so we don't error here with
+		 * g_return_if_fail().
+		 */
+		return;
+	}
 
 	removable_roots = tracker_storage_get_removable_device_roots (hal);
 #else
-	removable_roots = g_list_append (removable_roots, "/media");
-	removable_roots = g_list_append (removable_roots, "/mnt");
+	removable_roots = g_slist_append (removable_roots, "/media");
+	removable_roots = g_slist_append (removable_roots, "/mnt");
 #endif
 
+	flen = strlen (filename);
+
 	for (l = removable_roots; l; l = l->next) {
 		guint len;
 		
@@ -804,10 +810,10 @@ albumart_copy_to_local (TrackerStorage *hal,
 	}
 
 #ifdef HAVE_HAL
-	g_list_foreach (removable_roots, (GFunc) g_free, NULL);
+	g_slist_foreach (removable_roots, (GFunc) g_free, NULL);
 #endif
 
-	g_list_free (removable_roots);
+	g_slist_free (removable_roots);
 
 	if (on_removable_device) {
 		GFile *local_file, *from;
@@ -1057,8 +1063,9 @@ tracker_albumart_process (const unsigned char *buffer,
 			albumart_signal_queue_thumbnail (filename_uri, "image/jpeg");
 		}
 	} else {
-		g_debug ("Album art already exists for uri:'%s'", 
-			 filename_uri);
+		g_debug ("Album art already exists for uri:'%s' as '%s'", 
+			 filename_uri, 
+			 art_path);
 	}
 
 	if (local_uri && !g_file_test (local_uri, G_FILE_TEST_EXISTS)) {
